#!/usr/bin/env python

from os import pipe, system, path, fork
from sys import exit
from string import join,split,strip
from syslog import syslog, LOG_INFO
from threading import Thread, Lock
from xml.dom.minidom import Document
from MySQLdb import connect
import cPickle
from sss.server import SSSSelectServer, AddEvent, rquery

class nsm(SSSSelectServer):
    __implementation__='nsm'
    __component__='node-state-manager'

    def __parseinput__(self):
        self.readConf("/etc/nsm.conf")
        self.db = connect(host="localhost",db="statedb",user="nodeman",passwd="nm")
        self.cursor = self.db.cursor()
        self.nsdata=rquery('node-state-manager',self.cursor,'node-state','node-states')
        self.nsdata.SetAction('diagnose',['node'],self.Diagnose,['mode','level'])
        self.dispatch={'get-node-state':self.nsdata.Get,
                       'set-node-state':self.nsdata.Set,
                       'add-node-state':self.nsdata.Add,
                       'del-node-state':self.nsdata.Del,
                       'diagnose':self.nsdata.diagnose}

    def Diagnose(self,nodelist,mode,level):
        for (node,) in nodelist:
            self.runTest(node,level,mode)
            
    def readConf(self,filename):
        try:
            for data in open(filename,'r').readlines():
                (var,value)=split(data,'=')
                setattr(self,var,strip(value))
        except IOError, e:
            print "Unable to open the nsm.conf file: ", e
        
    def runTest(self,node,level,mode):
        #run the test, generate state based on exit code, update statedb, send event
        rc = system("%s %s %s %s"%(self.runcommand,node,level,mode))
        syslog(LOG_INFO, "%s %s %s %s"%(self.runcommand,node,level,mode))
        #the state may also be dependent on the level of the test that was run.
        if( rc != 0 ):
            print rc
            setclause = "adminstate='offline'"
        else:
            setclause = "adminstate='online'"
        whereclause = "Where node='%s'"%node
        query="UPDATE nodestate SET %s %s;"
        self.cursor.execute(query%(setclause,whereclause))
        AddEvent('node-state-manager','state-changed',node)
        

if __name__ == '__main__':
    from getopt import getopt, GetoptError
    from os import fork
    from sys import argv,exit
    try:
        (opts,args)=getopt(argv[1:],'d',['daemon='])
    except GetoptError,msg:
        print "%s\nUsage:\nnsm.py [-d] [--daemon <pidfile>]"%(msg)
        exit(1)
    daemon=filter(lambda x:(x[0] == '--daemon'),opts)
    ldebug=len((filter(lambda x:(x[0] == '-d'),opts)))
    if daemon:
        from sss.daemonize import daemonize
        daemonize(daemon[0][1])
    server=nsm(debug=ldebug)
    server.ServeForever()
